from flask import Flask, Response
from flask_cors import CORS
from requests_oauthlib import OAuth1Session, OAuth2Session
import os
import time
import json
from get_graphql_endpoint import endpoint
import threading
import random


app = Flask(__name__)
CORS(app)

twitterKeys = [
    'AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA',
    'AAAAAAAAAAAAAAAAAAAAAFXzAwAAAAAAMHCxpeSDG1gLNLghVe8d74hl6k4%3DRUMF4xAQLsbeBhTSRrCiQpJtxoGWeyHrDb5te2jpGskWDFW82F',
    'AAAAAAAAAAAAAAAAAAAAAAj4AQAAAAAAPraK64zCZ9CSzdLesbE7LB%2Bw4uE%3DVJQREvQNCZJNiz3rHO7lOXlkVOQkzzdsgu6wWgcazdMUaGoUGm',
]
privateKey = os.getenv('TWITTER_AUTH_KEY')
if privateKey:
    twitterKeys = [privateKey]

print("Booting")
ENDPOINT = endpoint()
print("Booted")

FeaturesDict = {
    "responsive_web_uc_gql_enabled": False, 
    "dont_mention_me_view_api_enabled": False,
    "interactive_text_enabled": False,
    "responsive_web_edit_tweet_api_enabled": False,
    "standardized_nudges_for_misinfo_nudges_enabled": False,
    "vibe_tweet_context_enabled": False,
    "standardized_nudges_misinfo": False,
    "responsive_web_enhance_cards_enabled": False,
    "vibe_api_enabled": False,
    "tweet_with_visibility_results_prefer_gql_limited_actions_policy_enabled": False,
    "responsive_web_text_conversations_enabled": False,
    "unified_cards_ad_metadata_container_dynamic_card_content_query_enabled": False,
    "graphql_is_translatable_rweb_tweet_is_translatable_enabled": False,
    "responsive_web_graphql_timeline_navigation_enabled": False,
    "tweetypie_unmention_optimization_enabled": False,
    "verified_phone_label_enabled": False,
    "responsive_web_twitter_blue_verified_badge_is_enabled": False,
    "view_counts_public_visibility_enabled": False,
    "view_counts_everywhere_api_enabled": False,
}
FeaturesJson = json.dumps(FeaturesDict)

healthLock = threading.Lock()
isHealthy = True

def getTwitterSession():
    twitter_b = OAuth2Session()
    twitter_b.headers["Authorization"] = "Bearer {}".format(random.choice(twitterKeys))
    return twitter_b

def doSearch(screen_name, twitter_b):
    global isHealthy
    searchurl_v2 = "https://api.twitter.com/2/search/adaptive.json"
    params_v2 = {"q": "from:@{}".format(screen_name), "count": 1, "spelling_corrections": 0, "tweet_search_mode": "live"}
    search_v2 = twitter_b.get(searchurl_v2, params=params_v2).json()
    isHealthy = 'globalObjects' in search_v2
    if isHealthy:
        return search_v2["globalObjects"]["tweets"]
    else:
        print("Rate limit occurred, marking as unavailable.")
        return None

@app.route('/_healthy')
def healthy():
    return {"healthy": True}
    #result = None
    #if isHealthy:
    #    return {'healthy': True}
    #with healthLock:
    #    if isHealthy:
    #        return {'healthy': True}
    #    doSearch('Dream', getTwitterSession())
    #    if isHealthy:
    #        print("Rate limit passed.")
    #        return {'healthy': True}
    #    return Response(status=500)
    #return result
    

@app.route("/<screen_name>")
def searchban(screen_name):
    print("Checking {}".format(screen_name))
    returnjson = {
                "timestamp": time.time(),
                "profile": {
                    # "id": "7080152",
                    # "screenName": "TwitterJP",
                    # "protected": False,
                    # "suspended": False,
                    # "has_tweets": True,
                    "exists": False,
                    "error": None,
                    "screen_name": screen_name,
                },
                # "check": {
                #     "search": 1484727214419628037,
                #     "suggest": True,
                #     "ghost": {"ban": True},
                #     "reply": {"ban": False, "tweet": "1480819689898987523", "in_reply_to": "1369626114381901828"}
                # }
            }

    
    # twitter = OAuth1Session(TWITTER_IPHONE_CK, TWITTER_IPHONE_CS)
    twitter_b = getTwitterSession()

    # check rate limit
    # response = twitter_b.get("https://api.twitter.com/1.1/application/rate_limit_status.json")
    # print(response.json())

    # profile_url = "https://api.twitter.com/1.1/users/show.json"
    # params = {"screen_name": screen_name}
    # profile_info = twitter_b.get(profile_url, params=params)
    # profile_json = profile_info.json()
    # print(profile_json)
    # if profile_info.status_code == 200:
    #     returnjson["profile"]["exists"] = True
    #     returnjson["profile"]["id"] = profile_json["id_str"]
    #     returnjson["profile"]["screenName"] = profile_json["screen_name"]
    #     returnjson["profile"]["protected"] = profile_json["protected"]
    # elif profile_info.status_code == 403:
    #     returnjson["profile"]["suspended"] = True
    #     return returnjson
    # else:
    #     returnjson["profile"]["error"] = profile_json["errors"][0]["message"]
    #     # return returnjson

    # if profile_json["protected"] == True:
    #     returnjson["profile"]["protected"] = True
    #     return returnjson

    # if profile_json["statuses_count"] == 0:
    #     returnjson["profile"]["has_tweets"] = False
    #     return returnjson
    # else:
    #     returnjson["profile"]["has_tweets"] = True

    # check whether the user has any tweets
    usertlurl = "https://api.twitter.com/1.1/statuses/user_timeline.json"
    params = {"screen_name": screen_name, "count": 200}

    usertl_b = twitter_b.get(usertlurl, params=params)
    usertl = usertl_b
    usertl_json = usertl.json()
    # # print(usertlb)
    # if "errors" in usertlb:
    #     return "An error occurred" # TODO: Better error handling
    # if len(usertlb) == 0:
    #     return "No tweets found"  # TODO: Better error handling

    if len(usertl_json) == 0:
        returnjson["profile"]["has_tweets"] = False
        return returnjson

    returnjson["profile"]["has_tweets"] = True

    if usertl.status_code == 200:
        returnjson["profile"]["exists"] = True
        returnjson["profile"]["id"] = usertl_json[0]["user"]["id_str"]
        returnjson["profile"]["screen_name"] = usertl_json[0]["user"]["screen_name"]
        # returnjson["profile"]["protected"] = usertl_json["protected"]
    elif usertl.status_code == 403:
        returnjson["profile"]["suspended"] = True
        return returnjson
    else:
        if "error" in usertl_json and usertl_json["error"] == "Not authorized.":
            returnjson["profile"]["protected"] = True
            returnjson["profile"]["suspended"] = True
            returnjson["profile"]["has_tweets"] = False
            return returnjson
        returnjson["profile"]["error"] = usertl_json
        return returnjson

    # if usertl_json["protected"] == True:
    #     returnjson["profile"]["protected"] = True  ## how do you determen protected and suspended
    #     return returnjson

    # if usertl_json["statuses_count"] == 0:
    #     returnjson["profile"]["has_tweets"] = False
    #     return returnjson
    # else:
    #     returnjson["profile"]["has_tweets"] = True

    returnjson["tests"] = {
        "search": True,    ## Search ban
        "typeahead": True, ## suggest ban
        "ghost": {"ban": None},
        "more_replies": {"ban": False, "tweet": "-1", "in_reply_to": "-1"}
    }

    # searchurl = "https://api.twitter.com/1.1/users/search.json"
    # params = {"q": "from:@{}".format(screen_name), "count": 1}
    # search = twitter_b.get(searchurl, params=params).json()
    # print(search)
    # if len(search) == 0:
    #     returnjson["test"]["search"] = "ban"
    #     return returnjson
    # else:
    #     return returnjson

    search_tweets = doSearch(screen_name, twitter_b)
    if search_tweets is None:
        returnjson["tests"]["search"] = '_error'
    elif search_tweets == {}:
        returnjson["tests"]["search"] = False
        # returnjson["tests"]["typeahead"] = False
    else:
        returnjson["tests"]["search"] = str(search_tweets[list(search_tweets.keys())[0]]["id"])

    
    suggestions = twitter_b.get("https://api.twitter.com/1.1/search/typeahead.json?src=search_box&result_type=users&q=@" + screen_name).json()
    try:
        returnjson["tests"]["typeahead"] = len([1 for user in suggestions["users"] if user["screen_name"].lower() == screen_name.lower()]) > 0
    except:
        returnjson["tests"]["typeahead"] = False

    if returnjson["tests"]["typeahead"] == True and returnjson["tests"]["search"] == "_error":
        returnjson["tests"]["search"] = "_implied_good"

    ## get replies
    ## Start GraphQL

    guest_session = twitter_b.post("https://api.twitter.com/1.1/guest/activate.json")

    twitter_b.headers["x-guest-token"] = guest_session.json()["guest_token"]

    user_id = returnjson["profile"]["id"]

    reply = None

    print("User ID of {}: {}".format(screen_name, user_id))

    get_reply_vars = { 
       "count": 700, "userId": user_id,
            "includePromotedContent": False, "withSuperFollowsUserFields": False, "withBirdwatchPivots": False,
            "withDownvotePerspective": False, "withReactionsMetadata": False,
             "withReactionsPerspective": False, "withSuperFollowsTweetFields": False, "withVoice": False, "withV2Timeline": False,
    }
    get_reply_param = param = {"variables": json.dumps(get_reply_vars), "features": FeaturesJson}
    replies = twitter_b.get("https://twitter.com/i/api/graphql/{}/{}".format(ENDPOINT["UserTweetsAndReplies"], "UserTweetsAndReplies"), params=get_reply_param)
    # print(replies.text)
    
    try:
        ghostban = True
        ghostTweetId = None
        ghostReplyId = None
        showmore = False
        showmoreTweetId = None
        showmoreReplyId = None
        repliesJson = replies.json()
        maindata = repliesJson["data"]["user"]["result"]["timeline"]["timeline"]["instructions"]
        for d in maindata:
            if d["type"] == "TimelineAddEntries":
                for ent in d["entries"]:
                    if ent["entryId"].startswith("tweet"):
                        tmp = ent["content"]["itemContent"]["tweet_results"]["result"]["legacy"]
                        if "in_reply_to_status_id_str" in tmp:
                            reply = tmp
                            tweetId = reply["in_reply_to_status_id_str"]
                            if ghostTweetId is None:
                                ghostTweetId = tweetId
                                ghostReplyId = reply["id_str"]
                            if showmoreTweetId is None:
                                showmoreTweetId = tweetId
                                showmoreReplyId = reply["id_str"]
                            tweet_detail_vars = {
                                "focalTweetId": tweetId,
                                "includePromotedContent":False,
                                "withBirdwatchNotes":False,
                                "withSuperFollowsUserFields":False,
                                "withDownvotePerspective":False,
                                "withReactionsMetadata":False,
                                "withReactionsPerspective":False,
                                "withSuperFollowsTweetFields":False,
                                "withVoice":False,
                            }
                            tweetdetails = twitter_b.get("https://twitter.com/i/api/graphql/{}/{}".format(ENDPOINT["TweetDetail"], "TweetDetail"), params={"variables": json.dumps(tweet_detail_vars), "features": FeaturesJson})
                            tweetData = tweetdetails.json()["data"]
                            if "threaded_conversation_with_injections" not in tweetData:
                                continue
                            insts = tweetdetails.json()["data"]["threaded_conversation_with_injections"]["instructions"]
                            for inst in insts:
                                if not ghostban and showmore:
                                    # all checks done
                                    break
                                if inst["type"] == "TimelineAddEntries":
                                    for ent in inst["entries"]:
                                        if not ghostban and showmore:
                                            # all checks done
                                            break
                                        print("Current entry of {} is: {}".format(screen_name, ent["entryId"]))
                                        if ent["entryId"].startswith("conversationthread") and ghostban:
                                            for item in ent["content"]["items"]:
                                                if "tweet_results" in item["item"]["itemContent"] and item["item"]["itemContent"]["tweet_results"]["result"]["legacy"]["user_id_str"] == user_id:
                                                    replyId = item["item"]["itemContent"]["tweet_results"]["result"]["legacy"]["id_str"]
                                                    returnjson["tests"]["ghost"] = {"ban": False, "tweet": tweetId, "in_reply_to": replyId}
                                                    ghostban = False
                                                    print("Found valid reply {} => {}, so {} is not ghost banned.".format(tweetId, replyId, screen_name))
                                                    break
                                        if ent["entryId"].startswith("cursor-showmorethreadsprompt") and not showmore:
                                            # showmore = True
                                            cursor_vars = tweet_detail_vars
                                            cursor_vars["cursor"] = ent["content"]["itemContent"]["value"]
                                            cursor = twitter_b.get("https://twitter.com/i/api/graphql/{}/{}".format(ENDPOINT["TweetDetail"], "TweetDetail"), params={"variables": json.dumps(cursor_vars), "features": FeaturesJson})

                                            cursor_insts = cursor.json()["data"]["threaded_conversation_with_injections"]["instructions"]
                                            for c_i in cursor_insts:
                                                if c_i["type"] == "TimelineAddEntries":
                                                    if len(c_i["entries"]) == 0:
                                                        returnjson["tests"]["more_replies"] = {"ban": True}
                                                        showmore = True
                                                        break
                                                    for c_ent in c_i["entries"]:
                                                        if c_ent["entryId"].startswith("conversationthread"):
                                                            print("Checking more contents of {} by {}".format(tweetId, screen_name))
                                                            for c_item in c_ent["content"]["items"]:
                                                                if c_item["item"]["itemContent"]["tweet_results"]["result"]["legacy"]["user_id_str"] == user_id:
                                                                    replyId = c_item["item"]["itemContent"]["tweet_results"]["result"]["legacy"]["id_str"]
                                                                    returnjson["tests"]["more_replies"] = {"ban": True, "in_reply_to": replyId, "tweet": tweetId}
                                                                    print("{} is reply deboosted because of {} => {}.".format(screen_name, tweetId, replyId))
                                                                    showmore = True
                                                                    break
        if ghostban:
            print("{} is ghost banned.".format(screen_name))
            returnjson["tests"]["ghost"] = {
                "ban": True,
                "tweet": ghostTweetId,
                "in_reply_to": ghostReplyId
            }
        if not showmore:
            returnjson["tests"]["more_replies"] = {
                "ban": False,
                "in_reply_to": showmoreReplyId,
                "tweet": showmoreTweetId
            }
        # No search ban || more replies => No ghost ban
        if returnjson["tests"]["search"] and returnjson["tests"]["search"] != "_error" and "ban" in returnjson["tests"]["ghost"] and returnjson["tests"]["ghost"]["ban"] == True or "ban" in returnjson["tests"]["more_replies"] and returnjson["tests"]["more_replies"]["ban"] == True:
            returnjson["tests"]["ghost"] = {
                "ban": False,
                "tweet": ghostTweetId,
                "in_reply_to": ghostReplyId
            }

        # No ghost ban && unknown more replies => No more replies ban
        if "ban" not in returnjson["tests"]["more_replies"] and "ban" in returnjson["tests"]["ghost"] and returnjson["tests"]["ghost"]["ban"] == False:
            returnjson["tests"]["more_replies"] = {
                "ban": False,
                "in_reply_to": showmoreReplyId,
                "tweet": showmoreTweetId
            }
    except KeyError as e:
        print("Errored testing {}".format(screen_name))
        print(e) 
        returnjson["tests"]["ghost"] = {}
        returnjson["tests"]["more_replies"] = {}


    #print("ban" in returnjson["tests"]["more_replies"])
    #print("ban" in returnjson["tests"]["ghost"])
    #print(returnjson["tests"]["ghost"])



    print("Result of {}: {}".format(screen_name, json.dumps(returnjson)))
    return returnjson

if __name__ == '__main__':
    app.run(debug=True, port=os.environ.get("PORT", 5000), host="0.0.0.0")
